class Organization():
    def __init__(self, **kwargs):
        self.id = kwargs.get('id')
        self.abraCustomerId = kwargs.get('abraCustomerId')
        self.addres = kwargs.get('address')
        self.hasActiveContract = kwargs.get('hasActiveContract')
        self.icoNumber = kwargs.get('icoNumber')
        self.isActual = kwargs.get('isActual')
        self.isCustomer = kwargs.get('isCustomer')
        self.name = kwargs.get('name')
        self.abraCode = kwargs.get('abraCode')
        self.xUcastnikId = kwargs.get('xUcastnikId')
        self.clientId = kwargs.get('clientId')
        self.updatedAt = kwargs.get('updatedAt')
        self.updatedBy = kwargs.get('updatedBy')
        self.isMember = kwargs.get('isMember')
        self.isCanceledMemberPart = kwargs.get('isCanceledMemberPart')
        self.masterOrganizationId = kwargs.get('masterOrganizationId')
        self.masterOrganizationAbraId = kwargs.get('masterOrganizationAbraId')
        self.masterOrganizationName = kwargs.get('masterOrganizationName')
        self.entityId = kwargs.get('entityId')
        self.isVip = kwargs.get('isVip')
        self.vipNote = kwargs.get('vipNote')
        self.organizationTypeId = kwargs.get('organizationTypeId')
        self.organizationTypeName = kwargs.get('organizationTypeName')
